
import base_section

from runtest import conditions

DNS_FEATURES_RELEASE_MAPPING = {
    'queens':
        {
            'api_v1': False,
            'api_v2': True,
            'bug_1573141_fixed': True,
            'api_v2_quotas': True,
            'api_v2_root_recordsets': True
        },
    'pike':
        {
            'api_v1': False,
            'api_v2': True,
            'bug_1573141_fixed': True,
            'api_v2_quotas': True,
            'api_v2_root_recordsets': True
        },
    'ocata':
        {
            'api_v1': False,
            'api_v2': True,
            'bug_1573141_fixed': True,
            'api_v2_quotas': True,
            'api_v2_root_recordsets': True
        },
    'newton':
        {
            'api_v1': False,
            'api_v2': True,
            'bug_1573141_fixed': True,
            'api_v2_quotas': True,
            'api_v2_root_recordsets': True
        },
    'mitaka':
        {
            'api_v1': False,
            'api_v2': True,
            'bug_1573141_fixed': True,
            'api_v2_quotas': True,
            'api_v2_root_recordsets': True
        }
}


class DnsFeatureEnabled(base_section.BaseSection):

    name = "dns_feature_enabled"
    options = [
        'api_admin',
        'api_v1',
        'api_v1_servers',
        'api_v2',
        'api_v2_quotas',
        'api_v2_root_recordsets',
        'bug_1573141_fixed',
    ]

    @property
    def api_admin(self):
        c = conditions.BaseRule('designate.server.enabled', 'eq', True, multiple='any')
        return self.get_item_when_condition_match('designate.server.admin_api.enabled', c)

    @property
    def api_v1(self):
        c = conditions.BaseRule('designate.server.enabled', 'eq', True, multiple='any')
        designate_version = self.get_item_when_condition_match('designate.server.version', c)
        if designate_version and designate_version in DNS_FEATURES_RELEASE_MAPPING:
            return DNS_FEATURES_RELEASE_MAPPING[designate_version]['api_v1']

    @property
    def api_v1_servers(self):
        pass

    @property
    def api_v2(self):
        c = conditions.BaseRule('designate.server.enabled', 'eq', True, multiple='any')
        designate_version = self.get_item_when_condition_match('designate.server.version', c)
        if designate_version and designate_version in DNS_FEATURES_RELEASE_MAPPING:
            return DNS_FEATURES_RELEASE_MAPPING[designate_version]['api_v2']

    @property
    def api_v2_quotas(self):
        c = conditions.BaseRule('designate.server.enabled', 'eq', True, multiple='any')
        designate_version = self.get_item_when_condition_match('designate.server.version', c)
        if designate_version and designate_version in DNS_FEATURES_RELEASE_MAPPING:
            return DNS_FEATURES_RELEASE_MAPPING[designate_version]['api_v2_quotas']

    @property
    def api_v2_root_recordsets(self):
        c = conditions.BaseRule('designate.server.enabled', 'eq', True, multiple='any')
        designate_version = self.get_item_when_condition_match('designate.server.version', c)
        if designate_version and designate_version in DNS_FEATURES_RELEASE_MAPPING:
            return DNS_FEATURES_RELEASE_MAPPING[designate_version]['api_v2_root_recordsets']

    @property
    def bug_1573141_fixed(self):
        c = conditions.BaseRule('designate.server.enabled', 'eq', True, multiple='any')
        designate_version = self.get_item_when_condition_match('designate.server.version', c)
        if designate_version and designate_version in DNS_FEATURES_RELEASE_MAPPING:
            return DNS_FEATURES_RELEASE_MAPPING[designate_version]['bug_1573141_fixed']
