
import base_section

from runtest import conditions


class NetworkFeatureEnabled(base_section.BaseSection):

    name = "network-feature-enabled"
    options = [
        'api_extensions',
        'floating_ips',
        'ipv6',
        'ipv6_subnet_attributes',
        'port_admin_state_change',
        'port_security',
    ]

    @property
    def api_extensions(self):
        # We will get this when running
        # tox -evenv -- tempest verify-config -uro tempest_config_file
        c = conditions.BaseRule(field='keystone.client.enabled', op='eq',
                                val=True)
        nodes = self.get_nodes_where_condition_match(c)
        if not nodes:
            return
        keystone_profile_admin = self.runtest_opts.get(
            'keystone_profile_admin', {})
        res = self.authenticated_openstack_module_call(
            nodes[0], 'neutronng.list_extensions',
            profile=keystone_profile_admin)[nodes[0]].get('extensions')
        if res:
            extensions = ', '.join([i['alias'] for i in res])
            return extensions

    @property
    def floating_ips(self):
        pass

    @property
    def ipv6(self):
        return conditions.BaseRule('*.neutron.server.enabled', 'eq', True,
                                   multiple='any').check(self.pillar)

    @property
    def ipv6_subnet_attributes(self):
        return conditions.BaseRule('*.neutron.server.enabled', 'eq', True,
                                   multiple='any').check(self.pillar)

    @property
    def port_admin_state_change(self):
        pass

    @property
    def port_security(self):
        c = conditions.BaseRule('neutron.server.enabled', 'eq', True)
        ext = self.get_item_when_condition_match(
            'neutron.server.backend.extension', c)
        if not ext:
            return
        if 'port_security' in ext:
            return ext['port_security'].get('enabled', False)

        return True
